/*
 * This file is part of PySide: Python for Qt
 *
 * Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
 *
 * Contact: PySide team <contact@pyside.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 */



#ifndef SBK_QTMULTIMEDIA_PYTHON_H
#define SBK_QTMULTIMEDIA_PYTHON_H

//workaround to access protected functions
#define protected public

#include <Python.h>
#include <conversions.h>
#include <pyenum.h>
#include <basewrapper.h>
#include <bindingmanager.h>
#include <memory>

#include <qsignal.h>
// Module Includes
#include <pyside_qtgui_python.h>
#include <pyside_qtcore_python.h>

// Binded library includes
#include <qabstractvideobuffer.h>
#include <qaudioengine.h>
#include <qaudioinput.h>
#include <qaudioengineplugin.h>
#include <qaudiodeviceinfo.h>
#include <qaudio.h>
#include <qaudiooutput.h>
#include <qvideoframe.h>
#include <qabstractvideosurface.h>
#include <qvideosurfaceformat.h>
#include <qaudioformat.h>
// Conversion Includes - Primitive Types
#include <QString>
#include <signalmanager.h>
#include <typeresolver.h>
#include <QTextDocument>
#include <QtConcurrentFilter>

// Conversion Includes - Container Types
#include <QMap>
#include <QStack>
#include <QVector>
#include <QStringList>
#include <QSet>
#include <QPair>
#include <pysideconversions.h>
#include <qqueue.h>
#include <QList>
#include <QMultiMap>

// Type indices
#define SBK_QAUDIO_IDX                                               0
#define SBK_QAUDIO_MODE_IDX                                          1
#define SBK_QAUDIO_ERROR_IDX                                         2
#define SBK_QAUDIO_STATE_IDX                                         3
#define SBK_QVIDEOFRAME_IDX                                          4
#define SBK_QVIDEOFRAME_FIELDTYPE_IDX                                5
#define SBK_QVIDEOFRAME_PIXELFORMAT_IDX                              6
#define SBK_QABSTRACTVIDEOBUFFER_IDX                                 7
#define SBK_QABSTRACTVIDEOBUFFER_MAPMODE_IDX                         8
#define SBK_QABSTRACTVIDEOBUFFER_HANDLETYPE_IDX                      9
#define SBK_QVIDEOSURFACEFORMAT_IDX                                  10
#define SBK_QVIDEOSURFACEFORMAT_YCBCRCOLORSPACE_IDX                  11
#define SBK_QVIDEOSURFACEFORMAT_DIRECTION_IDX                        12
#define SBK_QAUDIODEVICEINFO_IDX                                     13
#define SBK_QAUDIOFORMAT_IDX                                         14
#define SBK_QAUDIOFORMAT_SAMPLETYPE_IDX                              15
#define SBK_QAUDIOFORMAT_ENDIAN_IDX                                  16
#define SBK_QAUDIOENGINEFACTORYINTERFACE_IDX                         17
#define SBK_QABSTRACTAUDIOINPUT_IDX                                  18
#define SBK_QABSTRACTAUDIOOUTPUT_IDX                                 19
#define SBK_QABSTRACTAUDIODEVICEINFO_IDX                             20
#define SBK_QABSTRACTVIDEOSURFACE_IDX                                21
#define SBK_QABSTRACTVIDEOSURFACE_ERROR_IDX                          22
#define SBK_QAUDIOOUTPUT_IDX                                         23
#define SBK_QAUDIOINPUT_IDX                                          24
#define SBK_QAUDIOENGINEPLUGIN_IDX                                   25
#define SBK_QtMultimedia_IDX_COUNT                                   26

// This variable stores all python types exported by this module
extern PyTypeObject** SbkPySide_QtMultimediaTypes;

// Macros for type check

namespace Shiboken
{

// PyType functions, to get the PyObjectType for a type T
template<> inline PyTypeObject* SbkType<QAudio::Mode >() { return SbkPySide_QtMultimediaTypes[SBK_QAUDIO_MODE_IDX]; }
template<> inline PyTypeObject* SbkType<QAudio::Error >() { return SbkPySide_QtMultimediaTypes[SBK_QAUDIO_ERROR_IDX]; }
template<> inline PyTypeObject* SbkType<QAudio::State >() { return SbkPySide_QtMultimediaTypes[SBK_QAUDIO_STATE_IDX]; }
template<> inline PyTypeObject* SbkType<QVideoFrame::FieldType >() { return SbkPySide_QtMultimediaTypes[SBK_QVIDEOFRAME_FIELDTYPE_IDX]; }
template<> inline PyTypeObject* SbkType<QVideoFrame::PixelFormat >() { return SbkPySide_QtMultimediaTypes[SBK_QVIDEOFRAME_PIXELFORMAT_IDX]; }
template<> inline PyTypeObject* SbkType<QVideoFrame >() { return reinterpret_cast<PyTypeObject*>(SbkPySide_QtMultimediaTypes[SBK_QVIDEOFRAME_IDX]); }
template<> inline PyTypeObject* SbkType<QAbstractVideoBuffer::MapMode >() { return SbkPySide_QtMultimediaTypes[SBK_QABSTRACTVIDEOBUFFER_MAPMODE_IDX]; }
template<> inline PyTypeObject* SbkType<QAbstractVideoBuffer::HandleType >() { return SbkPySide_QtMultimediaTypes[SBK_QABSTRACTVIDEOBUFFER_HANDLETYPE_IDX]; }
template<> inline PyTypeObject* SbkType<QAbstractVideoBuffer >() { return reinterpret_cast<PyTypeObject*>(SbkPySide_QtMultimediaTypes[SBK_QABSTRACTVIDEOBUFFER_IDX]); }
template<> inline PyTypeObject* SbkType<QVideoSurfaceFormat::YCbCrColorSpace >() { return SbkPySide_QtMultimediaTypes[SBK_QVIDEOSURFACEFORMAT_YCBCRCOLORSPACE_IDX]; }
template<> inline PyTypeObject* SbkType<QVideoSurfaceFormat::Direction >() { return SbkPySide_QtMultimediaTypes[SBK_QVIDEOSURFACEFORMAT_DIRECTION_IDX]; }
template<> inline PyTypeObject* SbkType<QVideoSurfaceFormat >() { return reinterpret_cast<PyTypeObject*>(SbkPySide_QtMultimediaTypes[SBK_QVIDEOSURFACEFORMAT_IDX]); }
template<> inline PyTypeObject* SbkType<QAudioDeviceInfo >() { return reinterpret_cast<PyTypeObject*>(SbkPySide_QtMultimediaTypes[SBK_QAUDIODEVICEINFO_IDX]); }
template<> inline PyTypeObject* SbkType<QAudioFormat::SampleType >() { return SbkPySide_QtMultimediaTypes[SBK_QAUDIOFORMAT_SAMPLETYPE_IDX]; }
template<> inline PyTypeObject* SbkType<QAudioFormat::Endian >() { return SbkPySide_QtMultimediaTypes[SBK_QAUDIOFORMAT_ENDIAN_IDX]; }
template<> inline PyTypeObject* SbkType<QAudioFormat >() { return reinterpret_cast<PyTypeObject*>(SbkPySide_QtMultimediaTypes[SBK_QAUDIOFORMAT_IDX]); }
template<> inline PyTypeObject* SbkType<QAudioEngineFactoryInterface >() { return reinterpret_cast<PyTypeObject*>(SbkPySide_QtMultimediaTypes[SBK_QAUDIOENGINEFACTORYINTERFACE_IDX]); }
template<> inline PyTypeObject* SbkType<QAbstractAudioInput >() { return reinterpret_cast<PyTypeObject*>(SbkPySide_QtMultimediaTypes[SBK_QABSTRACTAUDIOINPUT_IDX]); }
template<> inline PyTypeObject* SbkType<QAbstractAudioOutput >() { return reinterpret_cast<PyTypeObject*>(SbkPySide_QtMultimediaTypes[SBK_QABSTRACTAUDIOOUTPUT_IDX]); }
template<> inline PyTypeObject* SbkType<QAbstractAudioDeviceInfo >() { return reinterpret_cast<PyTypeObject*>(SbkPySide_QtMultimediaTypes[SBK_QABSTRACTAUDIODEVICEINFO_IDX]); }
template<> inline PyTypeObject* SbkType<QAbstractVideoSurface::Error >() { return SbkPySide_QtMultimediaTypes[SBK_QABSTRACTVIDEOSURFACE_ERROR_IDX]; }
template<> inline PyTypeObject* SbkType<QAbstractVideoSurface >() { return reinterpret_cast<PyTypeObject*>(SbkPySide_QtMultimediaTypes[SBK_QABSTRACTVIDEOSURFACE_IDX]); }
template<> inline PyTypeObject* SbkType<QAudioOutput >() { return reinterpret_cast<PyTypeObject*>(SbkPySide_QtMultimediaTypes[SBK_QAUDIOOUTPUT_IDX]); }
template<> inline PyTypeObject* SbkType<QAudioInput >() { return reinterpret_cast<PyTypeObject*>(SbkPySide_QtMultimediaTypes[SBK_QAUDIOINPUT_IDX]); }
template<> inline PyTypeObject* SbkType<QAudioEnginePlugin >() { return reinterpret_cast<PyTypeObject*>(SbkPySide_QtMultimediaTypes[SBK_QAUDIOENGINEPLUGIN_IDX]); }

template<>
inline PyObject* createWrapper<QAbstractAudioInput >(const QAbstractAudioInput* cppobj, bool hasOwnership, bool isExactType)
{
    const char* typeName = 0;
    QAbstractAudioInput* value = const_cast<QAbstractAudioInput* >(cppobj);
    if (!isExactType)
        typeName = typeid(*value).name();
    PyObject* pyObj = Shiboken::SbkBaseWrapper_New(reinterpret_cast<SbkBaseWrapperType*>(SbkType<QAbstractAudioInput >()),value, hasOwnership, isExactType, typeName);
    PySide::signalUpdateSource(pyObj);
    return pyObj;
}
template<>
inline PyObject* createWrapper<QAbstractAudioOutput >(const QAbstractAudioOutput* cppobj, bool hasOwnership, bool isExactType)
{
    const char* typeName = 0;
    QAbstractAudioOutput* value = const_cast<QAbstractAudioOutput* >(cppobj);
    if (!isExactType)
        typeName = typeid(*value).name();
    PyObject* pyObj = Shiboken::SbkBaseWrapper_New(reinterpret_cast<SbkBaseWrapperType*>(SbkType<QAbstractAudioOutput >()),value, hasOwnership, isExactType, typeName);
    PySide::signalUpdateSource(pyObj);
    return pyObj;
}
template<>
inline PyObject* createWrapper<QAbstractAudioDeviceInfo >(const QAbstractAudioDeviceInfo* cppobj, bool hasOwnership, bool isExactType)
{
    const char* typeName = 0;
    QAbstractAudioDeviceInfo* value = const_cast<QAbstractAudioDeviceInfo* >(cppobj);
    if (!isExactType)
        typeName = typeid(*value).name();
    PyObject* pyObj = Shiboken::SbkBaseWrapper_New(reinterpret_cast<SbkBaseWrapperType*>(SbkType<QAbstractAudioDeviceInfo >()),value, hasOwnership, isExactType, typeName);
    PySide::signalUpdateSource(pyObj);
    return pyObj;
}
template<>
inline PyObject* createWrapper<QAbstractVideoSurface >(const QAbstractVideoSurface* cppobj, bool hasOwnership, bool isExactType)
{
    const char* typeName = 0;
    QAbstractVideoSurface* value = const_cast<QAbstractVideoSurface* >(cppobj);
    if (!isExactType)
        typeName = typeid(*value).name();
    PyObject* pyObj = Shiboken::SbkBaseWrapper_New(reinterpret_cast<SbkBaseWrapperType*>(SbkType<QAbstractVideoSurface >()),value, hasOwnership, isExactType, typeName);
    PySide::signalUpdateSource(pyObj);
    return pyObj;
}
template<>
inline PyObject* createWrapper<QAudioOutput >(const QAudioOutput* cppobj, bool hasOwnership, bool isExactType)
{
    const char* typeName = 0;
    QAudioOutput* value = const_cast<QAudioOutput* >(cppobj);
    if (!isExactType)
        typeName = typeid(*value).name();
    PyObject* pyObj = Shiboken::SbkBaseWrapper_New(reinterpret_cast<SbkBaseWrapperType*>(SbkType<QAudioOutput >()),value, hasOwnership, isExactType, typeName);
    PySide::signalUpdateSource(pyObj);
    return pyObj;
}
template<>
inline PyObject* createWrapper<QAudioInput >(const QAudioInput* cppobj, bool hasOwnership, bool isExactType)
{
    const char* typeName = 0;
    QAudioInput* value = const_cast<QAudioInput* >(cppobj);
    if (!isExactType)
        typeName = typeid(*value).name();
    PyObject* pyObj = Shiboken::SbkBaseWrapper_New(reinterpret_cast<SbkBaseWrapperType*>(SbkType<QAudioInput >()),value, hasOwnership, isExactType, typeName);
    PySide::signalUpdateSource(pyObj);
    return pyObj;
}
template<>
inline PyObject* createWrapper<QAudioEnginePlugin >(const QAudioEnginePlugin* cppobj, bool hasOwnership, bool isExactType)
{
    const char* typeName = 0;
    QAudioEnginePlugin* value = const_cast<QAudioEnginePlugin* >(cppobj);
    if (!isExactType)
        typeName = typeid(*value).name();
    PyObject* pyObj = Shiboken::SbkBaseWrapper_New(reinterpret_cast<SbkBaseWrapperType*>(SbkType<QAudioEnginePlugin >()),value, hasOwnership, isExactType, typeName);
    PySide::signalUpdateSource(pyObj);
    return pyObj;
}
// Generated converters declarations ----------------------------------

template<>
struct Converter<QAudio::Mode > : EnumConverter<QAudio::Mode >
{
};

template<>
struct Converter<QAudio::Error > : EnumConverter<QAudio::Error >
{
};

template<>
struct Converter<QAudio::State > : EnumConverter<QAudio::State >
{
};

template<>
struct Converter<QVideoFrame::FieldType > : EnumConverter<QVideoFrame::FieldType >
{
};

template<>
struct Converter<QVideoFrame::PixelFormat > : EnumConverter<QVideoFrame::PixelFormat >
{
};

template<>
struct Converter<QVideoFrame* > : ObjectTypeConverter<QVideoFrame >
{
};

template<>
struct Converter<QVideoFrame& > : ObjectTypeReferenceConverter<QVideoFrame >
{
};

template<>
struct Converter<QAbstractVideoBuffer::MapMode > : EnumConverter<QAbstractVideoBuffer::MapMode >
{
};

template<>
struct Converter<QAbstractVideoBuffer::HandleType > : EnumConverter<QAbstractVideoBuffer::HandleType >
{
};

template<>
struct Converter<QAbstractVideoBuffer* > : ObjectTypeConverter<QAbstractVideoBuffer >
{
};

template<>
struct Converter<QAbstractVideoBuffer& > : ObjectTypeReferenceConverter<QAbstractVideoBuffer >
{
};

template<>
struct Converter<QVideoSurfaceFormat::YCbCrColorSpace > : EnumConverter<QVideoSurfaceFormat::YCbCrColorSpace >
{
};

template<>
struct Converter<QVideoSurfaceFormat::Direction > : EnumConverter<QVideoSurfaceFormat::Direction >
{
};

template<>
struct Converter<QVideoSurfaceFormat > : ValueTypeConverter<QVideoSurfaceFormat >
{
};

template<>
struct Converter<QAudioDeviceInfo > : ValueTypeConverter<QAudioDeviceInfo >
{
};

template<>
struct Converter<QAudioFormat::SampleType > : EnumConverter<QAudioFormat::SampleType >
{
};

template<>
struct Converter<QAudioFormat::Endian > : EnumConverter<QAudioFormat::Endian >
{
};

template<>
struct Converter<QAudioFormat > : ValueTypeConverter<QAudioFormat >
{
};

template<>
struct Converter<QAudioEngineFactoryInterface* > : ObjectTypeConverter<QAudioEngineFactoryInterface >
{
};

template<>
struct Converter<QAudioEngineFactoryInterface& > : ObjectTypeReferenceConverter<QAudioEngineFactoryInterface >
{
};

template<>
struct Converter<QAbstractAudioInput* > : ObjectTypeConverter<QAbstractAudioInput >
{
};

template<>
struct Converter<QAbstractAudioInput& > : ObjectTypeReferenceConverter<QAbstractAudioInput >
{
};

template<>
struct Converter<QAbstractAudioOutput* > : ObjectTypeConverter<QAbstractAudioOutput >
{
};

template<>
struct Converter<QAbstractAudioOutput& > : ObjectTypeReferenceConverter<QAbstractAudioOutput >
{
};

template<>
struct Converter<QAbstractAudioDeviceInfo* > : ObjectTypeConverter<QAbstractAudioDeviceInfo >
{
};

template<>
struct Converter<QAbstractAudioDeviceInfo& > : ObjectTypeReferenceConverter<QAbstractAudioDeviceInfo >
{
};

template<>
struct Converter<QAbstractVideoSurface::Error > : EnumConverter<QAbstractVideoSurface::Error >
{
};

template<>
struct Converter<QAbstractVideoSurface* > : ObjectTypeConverter<QAbstractVideoSurface >
{
};

template<>
struct Converter<QAbstractVideoSurface& > : ObjectTypeReferenceConverter<QAbstractVideoSurface >
{
};

template<>
struct Converter<QAudioOutput* > : ObjectTypeConverter<QAudioOutput >
{
};

template<>
struct Converter<QAudioOutput& > : ObjectTypeReferenceConverter<QAudioOutput >
{
};

template<>
struct Converter<QAudioInput* > : ObjectTypeConverter<QAudioInput >
{
};

template<>
struct Converter<QAudioInput& > : ObjectTypeReferenceConverter<QAudioInput >
{
};

template<>
struct Converter<QAudioEnginePlugin* > : ObjectTypeConverter<QAudioEnginePlugin >
{
};

template<>
struct Converter<QAudioEnginePlugin& > : ObjectTypeReferenceConverter<QAudioEnginePlugin >
{
};

} // namespace Shiboken

// User defined converters --------------------------------------------
// Generated converters implemantations -------------------------------


#endif // SBK_QTMULTIMEDIA_PYTHON_H

